import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
/*
 * Dijkstra algorithm by Lars Vogel
 * Publicly available at http://www.vogella.com/articles/JavaAlgorithmsDijkstra/article.html
 * License: Eclipse Public License (Free to USE)
 * 
 *        
 */

public class Algorithm {
	public ArrayList<Edge> edges;
	static HashSet<Vertex> settledNodes = new HashSet<Vertex>();
	static HashSet<Vertex> unSettledNodes = new HashSet<Vertex>();
	static HashMap<Vertex, Vertex> predecessors = new HashMap<Vertex, Vertex>();
	static HashMap<Vertex, Integer> distance = new HashMap<Vertex, Integer>();

	public Algorithm(ArrayList<Edge> edges, Vertex source) {
		this.edges = edges;
		distance.put(source, 0);
		unSettledNodes.add(source);
		Vertex node;
		while (unSettledNodes.size() > 0) {
			node = getMinimum(unSettledNodes);
			settledNodes.add(node);
			unSettledNodes.remove(node);
			findMinimalDistances(node);
		}
	}

	private void findMinimalDistances(Vertex node) {
		ArrayList<Vertex> adjacentNodes = getNeighbors(node);
		int i = 0;
		while (i < adjacentNodes.size()) {
			if (getShortestDistance(adjacentNodes.get(i)) > getShortestDistance(node)
					+ getDistance(node, adjacentNodes.get(i))) {
				distance.put(adjacentNodes.get(i), getShortestDistance(node)
						+ getDistance(node, adjacentNodes.get(i)));
				predecessors.put(adjacentNodes.get(i), node);
				unSettledNodes.add(adjacentNodes.get(i));
			}
			i++;
		}

	}

	private int getDistance(Vertex node, Vertex target) {
		int i = 0;
		while (i < edges.size()) {
			if (edges.get(i).nachalo.equals(node)
					&& edges.get(i).krai.equals(target)) {
				return edges.get(i).weight;
			}
			i++;
		}
		throw new RuntimeException("    ");
	}

	private ArrayList<Vertex> getNeighbors(Vertex node) {
		ArrayList<Vertex> neighbors = new ArrayList<Vertex>();
		int i = 0;
		while (i < edges.size()) {
			if (edges.get(i).nachalo.equals(node)
					&& !settledNodes.contains(edges.get(i).krai)) {
				neighbors.add(edges.get(i).krai);
			}
			i++;
		}
		return neighbors;
	}

	private Vertex getMinimum(HashSet<Vertex> vertexes) {
		Vertex minimum = null;
		for (Vertex vert : vertexes) {
			if (minimum == null) {
				minimum = vert;
			} else {
				if (getShortestDistance(vert) < getShortestDistance(minimum)) {
					minimum = vert;
				}
			}
		}
		return minimum;
	}

	private int getShortestDistance(Vertex destination) {
		Integer d = distance.get(destination);
		return (d == null) ? Integer.MAX_VALUE : d;
	}

	public ArrayList<Vertex> getPath(Vertex target) {
		ArrayList<Vertex> path = new ArrayList<Vertex>();
		if (predecessors.get(target) == null) {
			return null;
		}
		path.add(target);
		while (predecessors.get(target) != null) {
			target = predecessors.get(target);
			path.add(target);
		}
		return path;
	}

}